#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cmath>

#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/program_options.hpp>
namespace po = boost::program_options;

#include "JobScheduling.h"

using namespace std;

void printUsageInformation();
void parseOptions(int argc, char *argv[], vector<AttributeFunction>& inputAttributes, vector<AttributeFunction>& outputAttributes, double& trainingRatio, double& validationRatio, double& testRatio, double& positivesRatio);
vector<AttributeFunction> createAttributesVector(string attributesString);

int main(int argc, char *argv[])
{
	srand(time(NULL));

    // read in file and create vector
	if(argc < 7)
	{
		printUsageInformation();
		exit(0);
	}


	vector<AttributeFunction> inputAttributeFunctions;
	vector<AttributeFunction> outputAttributeFunctions;

	// default
	double trainingRatio = 40, validationRatio = 30, testRatio = 30, positivesRatio = 0.5;

	parseOptions(argc,argv, inputAttributeFunctions, outputAttributeFunctions, trainingRatio, validationRatio, testRatio, positivesRatio);

	string inputFile = argv[argc-2];
	string outputFile = argv[argc-1];

	vector<JobData> jobs = parseFile(inputFile);
	cout << "num jobs =" << jobs.size() << endl;
	jobs = filterOutPartialJobs(jobs); // nijedan od njih 7 nema partial jobove
	cout << "num jobs =" << jobs.size() << endl;

	int numSamples = jobs.size();
	double sum = trainingRatio + validationRatio + testRatio;
	int trainingSetSize, validationSetSize, testSetSize;
	trainingSetSize = floor(trainingRatio/sum * numSamples + 0.5);
	validationSetSize = floor(validationRatio/sum * numSamples + 0.5);
	testSetSize = numSamples - trainingSetSize - validationSetSize; // what's left

	int trainingValidationSize =  changeTrainingSamplePositivesRatio(jobs, trainingSetSize + validationSetSize, positivesRatio);
	trainingSetSize = floor(trainingValidationSize * trainingRatio/(trainingRatio + validationRatio) + 0.5);
	validationSetSize = trainingValidationSize - trainingSetSize;

	if(int(jobs.size()) != trainingSetSize + validationSetSize + testSetSize)
	cerr << "Wrong number of samples" << endl;


	cout << "input size=" << inputSize(jobs, inputAttributeFunctions) << "    output size=" << outputSize(jobs, outputAttributeFunctions) << endl;
	cout << "Writing file..." << endl;
	createTrainingFile(jobs, inputAttributeFunctions, outputAttributeFunctions, outputFile, trainingSetSize, validationSetSize, testSetSize);
	cout << "File written" << endl;

    cout << "and... done." << endl;

    return 0;
}

void printUsageInformation()
{
	cout << "Usage: ./JobScheduling -i<input_features> -o<ouput_features> -r<training:validation:test> -pr<positives_ratio> input_file output_file" << endl;
}

void parseOptions(int argc, char *argv[], vector<AttributeFunction>& inputAttributes, vector<AttributeFunction>& outputAttributes, double& trainingRatio, double& validationRatio, double& testRatio, double& positivesRatio)
{
	po::options_description desc("Allowed options");
	desc.add_options()
	    ("-i", po::value<string>(), "all input attributes, comma delimited")
	    ("-o", po::value<string>(), "all output attributes, comma delimited")
	    ("-r", po::value<string>(), "data set ratios (training:validation:test)")
	    ("-p", po::value<double>(), "positives ratio in all samples - [0,1]")
	;

	po::variables_map vm;
	po::store(po::parse_command_line(argc, argv, desc), vm);
	po::notify(vm);

	if(vm.count("-i") != 1 || vm.count("-o") != 1 || vm.count("-r") != 1 || vm.count("-p") !=1)
	{
		cerr << "Error parsing options" << endl;
		exit(0);
	}

	inputAttributes = createAttributesVector(vm["-i"].as<string>());
	outputAttributes = createAttributesVector(vm["-o"].as<string>());

	vector<string> ratios;
	boost::split(ratios, vm["-r"].as<string>(), boost::is_any_of(":"));
	trainingRatio = strtod(ratios[0].c_str(),NULL);
	validationRatio = strtod(ratios[1].c_str(),NULL);
	testRatio = strtod(ratios[2].c_str(),NULL);

	positivesRatio = vm["-p"].as<double>();

}

vector<AttributeFunction> createAttributesVector(string attributesString)
{
	vector<AttributeFunction> attributes;

	vector<string> attribStrVec;
	boost::split(attribStrVec, attributesString, boost::is_any_of(","));
	for(size_t i=0;i<attribStrVec.size();i++)
		cout << "atrib[" << i << "]=" << attribStrVec[i] << endl;

	for(size_t i=0;i<attribStrVec.size();i++)
	{
		string str = attribStrVec[i];
		AttributeFunction func = NULL;

		if(str == "sT") func = submitTime;
		if(str == "wT") func = waitTime;
		if(str == "s") func = status;
		if(str == "rCPU") func = requestedNumberOfProcessors;
		if(str == "rT") func = requestedTime;
		if(str == "rM") func = requestedMemory;
		if(str == "uID") func = userID;
		if(str == "lsu") func = lastStatusOfUser;
		if(str == "lsa") func = lastStatusOfApplication;
		if(str == "lsua") func = lastStatusOfUserWithApplication;
		if(str == "lc") func = lastJobOfUserWithApplicationWasCanceled;
		if(str == "lrt") func = lastRunTimeOfUserWithApplication;
		if(str == "lCPUt") func = lastAverageCPUTimeOfUserWithApplication;
		if(str == "ea") func = elapsedTimeSinceUserLastUsedApplication;
		if(str == "ec") func = elapsedTimeSinceUserLastUsedCluster;

		if(func == NULL)
		{
			cerr << "Wrong attribute function" << endl;
			exit(1);
		}
		attributes.push_back(func);
	}

	return attributes;
}
