
#include <fstream>
#include <string>
#include <iostream>
#include <sstream>
#include <cstdlib>
#include <limits>
#include <cmath>
#include <vector>

#include "util.h"
#include "PSO.h"

using namespace std;



/** PSO minimizes fitnesses. */
ParticleSwarmOptimization::ParticleSwarmOptimization(PsoParameters& parameters, vector<float> &positions):
    params_(parameters), particlePositions_(positions)
{
    // set initial positions
    randomizeVector(particlePositions_,params_.lowerLimit,params_.upperLimit);
    iterations_ = 0;
    // preallocate vectors

    particleSpeeds_ = vector<float>(params_.populationSize * params_.nDimensions);

    bestParticlePositions_ = vector<float>(params_.populationSize * params_.nDimensions);
    bestParticleFitnesses_ = vector<float>(params_.populationSize);
    bestSwarmPosition_ = vector<float>(params_.nDimensions);

    for(int i=0;i<params_.populationSize;i++)
    {
        for(int j=0; j<params_.nDimensions;j++)
        {
            float range = params_.upperLimit - params_.lowerLimit;
            if(range <= 0) error("Wrong boundary",-113);
            float position = ( (float)rand()/RAND_MAX ) * range + params_.lowerLimit;
            float speed = ( (float)rand()/RAND_MAX ) * range * 2 - range;

            particlePositions_[i*params_.nDimensions+j] = position;
            particleSpeeds_[i*params_.nDimensions+j] = speed;
        }
        bestParticleFitnesses_[i] = numeric_limits<float>::max();
    }
    bestSwarmFitness_ = numeric_limits<float>::max();
}

vector<float>& ParticleSwarmOptimization::getPositions()
{
    return particlePositions_;
}
float ParticleSwarmOptimization::getSwarmFitness()
{
    return bestSwarmFitness_;
}
vector<float>& ParticleSwarmOptimization::getBestPosition()
{
    return bestSwarmPosition_;
}

/** Updates best fitnesses and positions for each particle and global fitness and position.
    v(i+1) = omega*v(i) + phiP*rP*(pk-xk) + phiG*rG*(g-xk)  == speed of k-th particle; r==random[0,1]; pk,g == best so far*/
void ParticleSwarmOptimization::updateFitnesses(vector<float> &fitnesses)
{
    if((int)fitnesses.size() != params_.populationSize) error("Wrong number of fitnesses.", -243);

    for(int i=0; i<params_.populationSize; i++)
    {
        if(fitnesses[i] < bestParticleFitnesses_[i])
        {
            int start = i*params_.nDimensions;
            int end = (i+1)*params_.nDimensions;
            copy(particlePositions_.begin() + start, particlePositions_.begin()+end, bestParticlePositions_.begin() + start);
            bestParticleFitnesses_[i] = fitnesses[i];
            if(fitnesses[i] < bestSwarmFitness_)
            {
                copy(particlePositions_.begin() + start, particlePositions_.begin()+end, bestSwarmPosition_.begin());
                bestSwarmFitness_ = fitnesses[i];
            }
        }
    }
}

void ParticleSwarmOptimization::updateSpeeds()
{
    for(size_t i=0;i<particleSpeeds_.size();i++)
    {
        float rP = (float)rand()/RAND_MAX;
        float rG = (float)rand()/RAND_MAX;

        particleSpeeds_[i] = params_.omega * particleSpeeds_[i] +
                             params_.phiP * rP * (bestParticlePositions_[i] - particlePositions_[i]) +
                             params_.phiG * rG * (bestSwarmPosition_[i % params_.nDimensions] - particlePositions_[i]);
        int maxSpeed = params_.maxSpeed;// * ( 1 - iterations_/ 60000.);
        if(abs(particleSpeeds_[i]) > maxSpeed)
            particleSpeeds_[i] = copysignf(maxSpeed, particleSpeeds_[i]);
    }

}

void ParticleSwarmOptimization::moveParticles()
{
    for(size_t i=0;i<particlePositions_.size();i++)
        particlePositions_[i] += particleSpeeds_[i];
}

void ParticleSwarmOptimization::nextIteration()
{
    iterations_++;
    updateSpeeds();
    moveParticles();
}



PsoParameters::PsoParameters()
{
    SetDefaults();
}

PsoParameters::PsoParameters(string fileName)
{
    SetDefaults();

    ifstream file(fileName.c_str(), ios::in);
    if(!file.is_open()) error(string("Error opening file: ") + fileName, -238);


    bool terminatingConditions = false; // were there terminating conditions before
    char line[101];
    while(file.getline(line,100))
    {
        //string line(cline);
        double result;

        if(ParseLine(line, "Population size", result))
            populationSize = (int) result;

        if(ParseLine(line, "omega", result))
            omega = result;

        if(ParseLine(line, "phiP", result))
            phiP = result;

        if(ParseLine(line, "phiG", result))
            phiG = result;

        if(ParseLine(line, "Lower limit", result))
            lowerLimit = result;

        if(ParseLine(line, "Upper limit", result))
            upperLimit = result;

        if(ParseLine(line, "Strip length", result))
            stripLength = result;

        // Terminating conditions
        if(ParseLine(line, "Iterations", result))
        {
            if(terminatingConditions == false) // this is the first
            {
                time = 0;
                maxFitness = 0;
                terminatingConditions = true;
            }
            iterations = result;
        }

        if(ParseLine(line, "Time", result))
        {
            if(terminatingConditions == false) // this is the first
            {
                iterations = 0;
                maxFitness = 0;
                terminatingConditions = true;
            }
            time = result;
        }

        if(ParseLine(line, "MSE", result))
        {
            if(terminatingConditions == false) // this is the first
            {
                iterations = 0;
                time = 0;
                terminatingConditions = true;
            }
            maxFitness = result;
        }

    }
    file.close();

}

void PsoParameters::SetDefaults()
{
    populationSize = 64;
    lowerLimit = -1; // position limits, same for all dimensions
    upperLimit = 1;
    maxSpeed = (upperLimit - lowerLimit);
    phiP = phiG = 1.49618;
    omega = 0.72984;

    stripLength = 10;

    time = 60;
    iterations = 0;
    maxFitness = 0;
}
void PsoParameters::PrintInfo()
{
    cout << "PSO parameters info:\n";
    cout << "  Population size: " << populationSize << endl;
    cout << "  Limits: [" << lowerLimit << "," << upperLimit << "]\n";
    cout << "  Max speed: " << maxSpeed << endl;
    cout << "  omega: " << omega << "   phiP: " << phiP << "   phiG: " << phiG << endl;
    cout << "  Time: " << time << endl;
    cout << "  Iterations: " << iterations << endl;
    cout << "  MSE: " << maxFitness << endl;
}
